--** Fullscreen RotoZoom effect in mode 19 by Mic.                            **--
--** This program is mainly a port of a Pascal program by Paul H Kahler.      **--


include bmp.e
include asm.e
include m19lib.e

--** Don't need this stuff... **--
without warning
without type_check

sequence cosTb,sinTb
sequence cosTb2,sinTb2
sequence img, pal
integer rotX,
        rot2,
        dr,
        dist,
        dd
atom vscreen
atom tm1,tm2,frames,rotY

--** There'll be 256 angles. **--
cosTb=repeat(0,256)
sinTb=cosTb
cosTb2=cosTb
sinTb2=cosTb


--** Rounds an atom to the nearest integer **--
function round(atom crap)
integer iCrap
 iCrap=floor(crap)
 if ((crap-iCrap)>0.5) then
  return (iCrap+1)
 else
  return iCrap
 end if
end function

--** Make trigonometric tables for later use. **--
procedure makeTables()
atom angle
atom vfix, vfix2
 vfix=(3.14159265/128)
 vfix2=(3.14159265/2)
 for i=1 to 256 do
  angle=(i-1)
  angle=(angle*vfix)
  sinTb[i]=round(sin(angle)*256)
  cosTb[i]=round(cos(angle)*256)
  --** Multiply by 1.2 to adjust for the non-square pixel ratio. **--
  sinTb2[i]=round(sin(angle+vfix2)*256*1.2)
  cosTb2[i]=round(cos(angle+vfix2)*256*1.2)
 end for
end procedure


--** Load a bitmap, set the graphics mode, blah, blah, blah... **--
img=load_bmp("rzoom.bmp")
--** img now is: {bitCount, address, width, height, palette} **--
vscreen=allocate(64000)
set_vga()
pal=floor(img[5]/4)
set_pal(pal)
pal=set_col(pal,0,{0,0,0})	 --** without this the screen border will be white, cos of the picture's palette **--	
img=img[2..4]
--** and now img is: {address, width, height} **--

--** Read-'n-Write variables for assembly code. **--
constant asm_i=allocate(4), asm_j=allocate(4)


--** This does the "magic" stuff (the drawing that is). **--
--** Presumes that the loaded image is 256*256. **--
constant rZoomCode=get_asm(
"pusha cld "&
"xor ebx,ebx "&
"mov eax,var_i "&
"mov [asm_i],eax "&
"mov eax,var_j "&
"mov [asm_j],eax "&
"mov esi,spriteAddr "&
"mov edi,screenAddr "&
"mov ecx,200 "&
"vLoop: "&
"push ecx "&
"mov eax,[asm_i] "&
"mov edx,[asm_j] "&
"mov ecx,320 "&
"hLoop: "&
"add eax,var_ddx "&
"add edx,var_ddy "&
"mov bh,dh "&
"mov bl,ah "&
"mov bl,[esi+ebx] "&
"mov [edi],bl "&
"inc edi "&
"loop hLoop "&
"mov eax,var_d2x "&
"mov edx,[asm_i] "&
"add eax,edx "&
"mov [asm_i],eax "&
"mov eax,var_d2y "&
"mov edx,[asm_j] "&
"add eax,edx "&
"mov [asm_j],eax "&
"pop ecx "&
"loop vLoop "&
"popa ret")
--** Constant values. **--
resolve_param("spriteAddr",rZoomCode,img[1])
resolve_param("screenAddr",rZoomCode,vscreen)
--** Read-'n-Write values. **--
resolve_param("asm_i",rZoomCode,asm_i)
resolve_param("asm_j",rZoomCode,asm_j)
--** Euphoria-write-, Asm-read values. **--
constant var_i=rZoomCode+get_param("var_i")
constant var_j=rZoomCode+get_param("var_j")
constant var_ddx=rZoomCode+get_param("var_ddx")
constant var_ddy=rZoomCode+get_param("var_ddy")
constant var_d2x=rZoomCode+get_param("var_d2x")
constant var_d2y=rZoomCode+get_param("var_d2y")



--** Do some precalculations and then call the assembly code. **--
procedure drawScreen(integer x,atom y,integer scale,integer rot)
integer temp
integer ddx,ddy,d2x,d2y
integer i,j
 temp=(cosTb[rot])
 temp=floor((temp*scale)/256)
 ddx=temp
 temp=(sinTb[rot])
 temp=floor((temp*scale)/256)
 ddy=temp
 temp=(cosTb2[rot])
 temp=floor((temp*scale)/256)
 d2x=temp
 temp=(sinTb2[rot])
 temp=floor((temp*scale)/256)
 d2y=temp
 i=x-(ddx*160)-(d2x*100)
 j=y-(ddy*160)-(d2y*100)
 poke4(var_i,i)
 poke4(var_j,j)
 poke4(var_ddx,ddx)
 poke4(var_ddy,ddy)
 poke4(var_d2x,d2x)
 poke4(var_d2y,d2y)
 call(rZoomCode)
end procedure



--** MAIN Code **--
rotX=32768
rotY=0
rot2=1				 --** Rotation angle **--
dr=1				 --** Rotation speed **--
dist=1000			 --** Distance **--
dd=65526			 --** "zomming-speed". Lower value gives faster zooming, higher values gives slower zomming **--
makeTables()
frames=0			 --** initialize frame counter **--	
tm1=time()			 --** Get starting time **--

--** MAIN Loop **--
while get_key()=-1 do
 drawScreen(rotX,rotY,dist,rot2+1) --** Draw the stuff to the buffer (vscreen). **--
 wait_vrt()			 --** Wait for vertical retrace. **--
 asm_copy_screen(vscreen,#A0000) --** Copy buffer to real screen. **--
 frames=frames+1
 rot2=rot2+dr
 rotY=rotY+128
 dist=dist+dd
 dist=and_bits(dist,#FFFF)
 if (dist>=1800) or (dist<=2) then
  dd=-dd
 end if
 if rand(150)=1 then		 
  dr=rand(7)-3			 --** Randomize the rotation speed. **--
 end if
 rot2=and_bits(rot2,#00FF)	 --** in case the angle gets too small/large **--
end while
--** End of Main Loop **--

tm2=time()			 --** Get ending time. **--

pal=fade_to_rgb(pal,{0,0,0},2)	 --** fade to black **--

--** Restore graphics mode and free all allocated memory **--
restore_graphics()		 
free_asm_code()   		 
free(vscreen)
free(img[1])
free(rZoomCode)
free(asm_i)
free(asm_j)

--** Print fps **--
print(1,floor(frames/(tm2-tm1)))
puts(1," fps\n")
--** End of Main Code **--
